/**
  * @file BrushlessFT90.c
  * @brief This module contains the implementation of the brushless hardware
  * functions.

  * @author Stefan Popovic
  *
  * @date 11/09/2016
  *
  * @version 0.1a
  *
  * @copyright GNU Public License
  *
  * Test configuration:
  *
  * MCU: FT900
  * Dev.Board: EasyFT90x v7 
  * Oscillator: HS-PLL 100 MHz
  * Ext. Modules: Brushless Click Board
  * SW: MikroC PRO for FT90x v2.0.0

  * NOTES:
    - Place Brushless click board on mikroBUS slot 1.
    - Turn ON the GPIO_PORT_56_66 LEDs at SW8.8.
    - Pull-up switch SW1.4 (VCC position) and
    - Pull-down pins GP27 and GP31 at GPIO_PORT_24_31
    - PWM: GP56  CONFIG_CH0
    - Vsense: GP09
    - Int: GP03

  /**************************************************************************
  * Module Preprocessor Constants
  **************************************************************************/
  #define CW    0x01
  #define CCW   0x00
  #define SKIP  5

   /**************************************************************************
  * Module Preprocessor Macros
  **************************************************************************/


  /**************************************************************************
  * Module Typedefs
  **************************************************************************/


  /**************************************************************************
  * Includes
  **************************************************************************/
  #include "built_in.h"
  #include "recources.h"

  /**************************************************************************
  * Module Variable Definitions
  **************************************************************************/
   sbit MOTOR_DIR at GPIO_PIN1_bit;
   
  // TFT module connections
  char TFT_DataPort at GPIO_PORT_32_39;
  sbit TFT_RST at GPIO_PIN42_bit;
  sbit TFT_RS at GPIO_PIN40_bit;
  sbit TFT_CS at GPIO_PIN41_bit;
  sbit TFT_RD at GPIO_PIN46_bit;
  sbit TFT_WR at GPIO_PIN47_bit;
  // End TFT module connections

  unsigned int current_speed;
  unsigned int pwm_period;
  char txt[24];

  // MCU initialization
  void MCU_Init() 
  {
    GPIO_Digital_Input (&GPIO_PORT_24_31, _GPIO_PINMASK_3 | _GPIO_PINMASK_7);
    GPIO_Digital_Input(&GPIO_PORT_48_55, _GPIO_PINMASK_4);
    GPIO_Digital_Input(&GPIO_PORT_08_15, _GPIO_PINMASK_1);
    GPIO_Digital_Input(&GPIO_PORT_00_07, _GPIO_PINMASK_1);

    ADC_Set_Input_Channel(_ADC_CHANNEL_4 | _ADC_CHANNEL_5); // ADC channels 4 and 5
    ADC_Init();
    Delay_ms(100);
    current_speed = 19000; //initial value for current duty. With lower duty, BLCD has higher rpm.

    pwm_period = PWM_Init(5000); // Initialize PWM2 module at 5kHz
    PWM_ConfigChannel(current_speed, _PWM_INIT_STATE_0, _PWM_CHANNEL_0); // Set current duty for PWM2 on channel 0
    PWM_EnableOutput();
  }

  // Draw graphical frame
  void DrawFrame() {
    TFT_Fill_Screen(CL_TEAL);
    TFT_Set_Pen(CL_BLACK, 1);
    TFT_Set_Brush(1,CL_TEAL, 0, 0, CL_TEAL, CL_TEAL);
    TFT_Line(20, 220, 300, 220);
    TFT_Line(20,  46, 300,  46);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("Brushless  Click  Board  Example", 12, 14);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("EasyFT90x v7", 19, 223);
    TFT_Set_Font(&Verdana12x13_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("www.mikroe.com", 200, 223);
    TFT_Set_Font(&HandelGothic_BT21x22_Regular, CL_BLACK, FO_HORIZONTAL);
    TFT_Write_Text("PWM  DUTY", 10, 80 );
    TFT_Write_Text("PWM  MAX", 10, 140);
    WordToStr(pwm_period, txt);
    TFT_Write_Text(txt, 150, 140);
    TFT_Set_Pen(CL_TEAL, 1);
  }

  void main() 
  {
    MCU_Init();
    TFT_Init_ILI9341_8bit(320, 240); // TFT Initialization
    DrawFrame();
    MOTOR_DIR = CCW;     // Setting motor direction to counter-clock-wise

    while(1)
    {

      if (GPIO_PIN27_bit)                 // button on GPIO27 pressed
      {
          Delay_ms(1);
          current_speed = current_speed + SKIP;
          if (current_speed > pwm_period)       // if we increase current_duty greater then possible pwm_period value
          {
           current_speed = 0;                    // reset current_duty value to zero
          }

          PWM_ConfigChannel(current_speed, _PWM_INIT_STATE_0, _PWM_CHANNEL_0);   // set newly acquired duty ratio
       }

      if (GPIO_PIN31_bit)                 // button on GPIO31 pressed
       {
          Delay_ms(1);
          current_speed = current_speed - SKIP;
          if (current_speed > pwm_period)       // if we decrease current_duty greater then possible pwm_period value (overflow)
          {
             current_speed = pwm_period;          // set current duty to max possible value
          }
          PWM_ConfigChannel(current_speed, _PWM_INIT_STATE_0, _PWM_CHANNEL_0);
       }

      Delay_ms(20);

      // current_speed display
      WordToStr(current_speed, txt);
      TFT_Rectangle(150, 80, 250, 110);
      TFT_Write_Text(txt, 150, 80);

  }

}